<?php

namespace Newgenpower\Project\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Newgenpower\Project\Forms\Feature as FeatureForm;
use Newgenpower\Project\Http\Requests\FeatureRequest;
use Newgenpower\Project\Interfaces\FeatureRepositoryInterface;
use Newgenpower\Project\Repositories\Eloquent\Filters\FeatureResourceFilter;
use Newgenpower\Project\Repositories\Eloquent\Presenters\FeatureListPresenter;

/**
 * Resource controller class for feature.
 */
class FeatureResourceController extends BaseController
{

    /**
     * Initialize feature resource controller.
     *
     *
     * @return null
     */
    public function __construct(FeatureRepositoryInterface $feature)
    {
        parent::__construct();
        $this->form = FeatureForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('newgenpower.project.modules'), 'project', guard_url('project'));
        $this->repository = $feature;
    }

    /**
     * Display a list of feature.
     *
     * @return Response
     */
    public function index(FeatureRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(FeatureResourceFilter::class)
            ->setPresenter(FeatureListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('project::feature.names'))
            ->view('project::feature.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display feature.
     *
     * @param Request $request
     * @param Model   $feature
     *
     * @return Response
     */
    public function show(FeatureRequest $request, FeatureRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('project::feature.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('project::feature.show')
            ->output();
    }

    /**
     * Show the form for creating a new feature.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(FeatureRequest $request, FeatureRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('project::feature.name'))
            ->view('project::feature.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new feature.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FeatureRequest $request, FeatureRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('project::feature.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('project/feature/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/project/feature'))
                ->redirect();
        }

    }

    /**
     * Show feature for editing.
     *
     * @param Request $request
     * @param Model   $feature
     *
     * @return Response
     */
    public function edit(FeatureRequest $request, FeatureRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('project::feature.name'))
            ->view('project::feature.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the feature.
     *
     * @param Request $request
     * @param Model   $feature
     *
     * @return Response
     */
    public function update(FeatureRequest $request, FeatureRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('project::feature.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('project/feature/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('project/feature/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the feature.
     *
     * @param Model   $feature
     *
     * @return Response
     */
    public function destroy(FeatureRequest $request, FeatureRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('project::feature.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('project/feature/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('project/feature/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
